/*******************************************************************************
 *	MainGui.cc	tmsSigGen GUI process
 *			T. Barnaby,	BEAM Ltd,	2007-01-30
 *******************************************************************************
 */

#include <stdio.h>
#include <syslog.h>
#include <unistd.h>
#include <signal.h>
#include <sys/resource.h>
#include <Gui.h>

void usage(){
	fprintf(stderr, "Usage: tmsSigGen\n");
}

int main(int argc, char** argv){
	BError		err;
	int		a;
	BString		s;

	QApplication*	app;
	Gui*		gui;
	
	for(a = 1; a < argc; a++){
		if(argv[a][0] == '-'){
			switch(argv[a][1]){
			default:	usage();	return 1;
			}
		}
	}
	
	app = new QApplication(argc, argv);
	gui = new Gui();
	gui->init("TmsSigGen");
	app->setMainWidget(gui);
	gui->show();

	app->exec();

	return 0;
}
