/*
* Title:	TestWin.h
* Author:	M.Thomas BEAM Ltd
* Date:		2007-02-13
*
* Contents:	User interface to the TMS server selftest.
*
* Mod Rec:
*
*/

#ifndef TESTWIN_H
#define TESTWIN_H

#include <Control.h>
#include <BString.h>
#include <qhbox.h>
#include <qtextedit.h>

/*! Configure System and setup user interface window. 
*
*/
class TestWin : public QHBox {
        Q_OBJECT
public:
        TestWin(QWidget* w,Control& c);
	~TestWin();
public slots:

private slots:
	void	show();				///< Update and make the widget visible
	void	runTest();			///< Run the remote server seft test

private:
	void	warningDialog(BString title, BError err);

	QTextEdit*		oresults;
	Control&		ocontrol;
};
#endif
