/*
* Title:	Debug.h 
* Author:	M.Thomas BEAM Ltd
* Date:		2007-04-02
*
* Contents:	Debug support
*
* Mod Rec:
*
*/
#ifndef DEBUG_H
#define DEBUG_H

#define DBG_STD                 0x000001			
#define DBG_CONTROL             0x000002	// Debug access to the TMS server		

extern int      gdebug;

#define dprintf(level, fmt, a...)       if((level) & gdebug) printf(fmt, ##a);

#endif
