/*******************************************************************************
 *	Client.cc	HomeNet RPC protocol test client
 *			T.Barnaby,	BEAM Ltd,	9/5/03
 *******************************************************************************
 */
#include <stdio.h>
#include <stdlib.h>
#include <BString.h>
#include <Test1C.h>

int main(int argc, char** argv){
	BNetAddress	nadd("", 11000);
	HomeNet::Test1	client;
	BError		err;
	int		r;
	Int32		a1;
	HomeNet::Data	a2;
	Int32		a3;
	
	if(err = client.connect(nadd)){
		cerr << err.getString() << "\n";
		return 1;
	}
	
	r = client.getInfo(a1, a2, a3);
	
	cout << "Result: " << r << "\n";
	
	return 0;
}
