/*******************************************************************************
 *	BHtml		HTML Generation classes
 *			T.Barnaby,	BEAM Ltd,	2006-09-03
 *******************************************************************************
 */

#include <BString.h>
#include <BList.h>

#ifndef BHtml_H
#define BHtml_H 1

class BHtml {
public:
			BHtml(BString tag = "", BString args = "", BString text = "");
			~BHtml();
			
	BHtml*		append(BHtml node);
	BHtml*		append(BString tag = "", BString args = "", BString text = "");
	BHtml*		appendText(BString text);
	
	BString		render();

private:
	BString		tag;
	BString		args;
	BList<BHtml>	nodes;
};
#endif
