/*******************************************************************************
 *	BHtmlPage		HTML Pages
 *				T.Barnaby,	BEAM Ltd,	2006-09-04
 *******************************************************************************
 */
#ifndef BHtmlPage_H
#define BHtmlPage_H	1

#include <BHtml.h>

class BHtmlPage {
public:
			BHtmlPage();
			~BHtmlPage();
	
	void		setTitle(BString title);
	void		setLeftSide(BString leftSide);
	void		setContent(BString content);

	BString		render();

	BString		otitle;	
	BString		oleftSide;	
	BString		ocontent;
};
#endif
