/*******************************************************************************
 *	Client.cc	HomeNet RPC protocol test client
 *			T.Barnaby,	BEAM Ltd,	9/5/03
 *******************************************************************************
 */
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <BString.h>
#include <TestD.h>
#include <TestC.h>
#include <TestS.h>


int main(int argc, char** argv){
	Test::LightCallback	client;
	BError			err;

	if(err = client.connectService("testCallback")){
		std::cerr << "Connection Error: " << err.getString() << "\n";
		return 1;
	}
	printf("Test1\n");
	err = client.setLevel(22);

	return 0;
}
